# -*- coding: iso_8859_1 -*-

from aspylib import astro, photometry, conf
import numpy as np
import time, os, datetime
import matplotlib.pyplot as plt
import pp

#------------------------------------------------------------------------------------
#		Input parameters (basic)
#------------------------------------------------------------------------------------

#--- input frames ---
Folder = u"C:\\images\\"
Imagelist = [Folder + "images-calibrated-" + str(i+1) + ".fit" for i in range(100)]

#--- for detecting stars ---
target_nb = 2000

#--- disk size for photometry ---
sizes = [[3,8,24]]

#--- output filenames ---
generic_name = Folder + 'target_'

#--- semi-size of square areas used for star max detection and star fitting ---
psf_fit_area = 8			#typically a few times (2-3) the star FWHM
max_detection_area = 2

#------------------------------------------------------------------------------------
#		Input parameters (advanced)
#------------------------------------------------------------------------------------

#--- names of output files ---
outputname1 = generic_name + "01_check_images.txt"
outputname2 = generic_name + "02_image_shifts.txt"
outputname3 = generic_name + "03_magnitudes.txt"

#--- photometry method ---
#method = 'maximum'		#--> photometric disk centered at pixel with max value
method = 'centroid'		#--> photometric disk centered at star centroid (most accurate method)

#####################################################################################
#	SCRIPT STARTS HERE
#####################################################################################
#------------------------------------------------------------------------------------
#		Automatic detection of stars
#------------------------------------------------------------------------------------
print
print '*********************************************************************'
print '      SCRIPT FOR AUTOMATIC SEARCH OF VARIABLE STARS'
print '*********************************************************************'

print
print '---------------------------------------------------------------------'
print '      Step 1 : automatic star detection'
print '---------------------------------------------------------------------'
print

#--- finding stars on first image ---
astro.maximize_window(u"C:\\WINDOWS\\system32\\cmd.exe")
sx, sy = astro.get_window_size(u"C:\\WINDOWS\\system32\\cmd.exe")
image0 = astro.get_imagedata([Imagelist[0]])
xy0 = astro.find_stars(image0, target_nb, detection_area = max_detection_area, saturation = [True,conf.saturation])

#--- plotting detected stars ---
fig, ax = astro.display(image0, title="First image")
ax.scatter(xy0[:,1],xy0[:,0],marker='+',color='g')
plt.sca(ax)
plt.xlim([0,image0.shape[1]])
plt.ylim([0,image0.shape[0]])
plt.draw()
fx, fy = astro.get_window_position("First image")
astro.move_window("First image", [sx, fy])
astro.set_focus(u"C:\\WINDOWS\\System32\\cmd.exe")
raw_input("press Enter if ok")
plt.close()

#------------------------------------------------------------------------------------
#	selects images
#------------------------------------------------------------------------------------

#------ checking quality of measured frames ------

print
print '---------------------------------------------------------------------'
print '      Step 2 : checking the input images'
print '---------------------------------------------------------------------'
print
Imagelist2 = photometry.check_pictures_quality(Imagelist, outputname1)

#------ calculating shifts ------

print
print '---------------------------------------------------------------------'
print '      Step 3 : calculation of image shifts'
print '---------------------------------------------------------------------'
print
Imagelist3, shifts = photometry.calculate_shifts(Imagelist2, outputname2, detection_area = psf_fit_area, nb_stars = 12)

#------------------------------------------------------------------------------------
#	calculates instrumental magnitudes
#------------------------------------------------------------------------------------
t0 = time.time()
#------ date information ------
dateobs = astro.julian_day(astro.get_headervalues(Imagelist3, 'DATE-OBS'))

#------ perform aperture photometry ------
print
print '---------------------------------------------------------------------'
print '      Step 4 : calculating aperture photometry'
print '---------------------------------------------------------------------'
print

#------------------------------------------------------------------------------
#xy = []
#mag = []
# for i in range(len(Imagelist3)):
	# imagei = astro.get_imagedata([Imagelist3[i]])
	# xy.append(xy0 + np.round(shifts[i,:]))
	# if method == 'maximum':
		# xy[-1] = astro.max_stars(imagei, xy[-1], max_detection_area, error_messages = 2)
		# mag.append(photometry.aperture_photometry(imagei, sizes, xy[-1], method = 'maximum', error_messages = 2))
	# elif method == 'centroid':
		# xy[-1] = astro.centroid_stars(imagei, xy[-1], max_detection_area, error_messages = 2)
		# mag.append(photometry.aperture_photometry(imagei, sizes, xy[-1], method = 'centroid', error_messages = 2))
# xy = np.array(xy)
# mag = np.array(mag)
#------------------------------------------------------------------------------

# tuple of all parallel python servers to connect with
job_server = pp.Server()
print "Starting pp with", job_server.get_ncpus(), "workers"

xy = []
mag = []

def parallel_photometry(i, Imagelist, xy, sizes, max_detection_area, method, error_messages):
	imagei = aspylib.astro.get_imagedata([Imagelist[i]])
	if method == 'maximum':
		xy = aspylib.astro.max_stars(imagei, xy, max_detection_area, error_messages = error_messages)
	elif method == 'centroid':
		xy = aspylib.astro.centroid_stars(imagei, xy, max_detection_area, error_messages = error_messages)
	mag = aspylib.photometry.aperture_photometry(imagei, sizes, xy, method = method, error_messages = error_messages)
	return xy, mag

jobs = []
for i in range(len(Imagelist3)):	
	jobs.append(job_server.submit(parallel_photometry,(i, Imagelist3, xy0 + np.round(shifts[i,:]), sizes, max_detection_area, "centroid", 2,), (), ("aspylib.astro", "aspylib.photometry",)))

for job in jobs:
	out = job()
	xy_out, mag_out = out
	xy.append(xy_out)
	mag.append(mag_out)

xy = np.array(xy)
mag = np.array(mag)
#------------------------------------------------------------------------------

print time.time()-t0, "seconds"

#*********************************************************************************************************
#		Calculating statistics
#*********************************************************************************************************

print
print '---------------------------------------------------------------------'
print '      Step 5 : processing results'
print '---------------------------------------------------------------------'
print

diff_mag, stats_diff, quality_flag, candid_ref = photometry.compute_statistics(mag[:,:,0], plotting = True)
#---> statistics calculated only for first disk

#*********************************************************************************************************
#		Exporting photometry results
#*********************************************************************************************************

print
print '---------------------------------------------------------------------'
print '      Step 6 : writing results in output file'
print '---------------------------------------------------------------------'
print

photometry.write_auto_magnitudes(Imagelist3, dateobs, xy, sizes[0], diff_mag, stats_diff, quality_flag, candid_ref, outputname3)
#---> output file written only for first disk

#*********************************************************************************************************
#		Displaying statistics
#*********************************************************************************************************

raw_input("finished")