# -*- coding: iso_8859_1 -*-

from aspylib import astro, conf
import numpy as np
import calib7

def calibrate_coalign_and_save(Imagelist, Darklist, Flat, Imagelist2, Config, detection_area=10, nb_stars=12, excl=[], incl=[], text=False, Headers={}):

	#------ creating dark frame ------
	darks = astro.get_imagedata(Darklist)
	dark_final = astro.add_median(darks)
	hotpixels = calib7.find_hotpixels(dark_final, 500)
	
	#------ creating flat frame ------
	flat = astro.get_imagedata([Flat])
	flat = flat / astro.get_median(flat)
	flat = flat + 1e-10

	#------ calibration of 1st image ------
	image0 = astro.get_imagedata([Imagelist[0]])
	image0_calib = (image0 - dark_final) / flat
	image0_avg = (np.roll(image0_calib,1,axis=0) + np.roll(image0_calib,-1,axis=0) + np.roll(image0_calib,1,axis=1) + np.roll(image0_calib,-1,axis=1)) / 4.
	image0_calib = image0_avg*hotpixels + image0_calib*(1.-hotpixels)

	#------ checking for saturation ------
	test = (image0_calib<conf.saturation-0.001)*(image0!=conf.saturation)
	image0_calib = (1.-test)*conf.saturation + test*image0_calib
	image0_calib = (image0_calib>=0.)*image0_calib							#this step is required for coalignment with star matching (for star detection) otherwise mean is too low due to values << 0
	print 'dark and flat calibration ... done'
	print 'shifts (vert, hor) wrt reference image:', 0., 0.
	
	#save image
	headerdict = astro.create_headers(Config, [Imagelist[0]])[0]
	headerdict.update(Headers)
	astro.save_imagelist(image0_calib, [headerdict], [Imagelist2[0]])
	print Imagelist2[0] + ' saved'

	#------ same for all other images ------
	stars0 = astro.find_stars_accurate(image0_calib, nb_stars, detection_area, excluded=excl, included=incl)
	for i in range(1,len(Imagelist)):

		#loading and calibration
		imagei = astro.get_imagedata([Imagelist[i]])
		imagei_calib = (imagei - dark_final) / flat
		imagei_avg = (np.roll(imagei_calib,1,axis=0) + np.roll(imagei_calib,-1,axis=0) + np.roll(imagei_calib,1,axis=1) + np.roll(imagei_calib,-1,axis=1)) / 4.
		imagei_calib = imagei_avg*hotpixels + imagei_calib*(1.-hotpixels)
	
		#checking for saturation
		test = (imagei_calib<conf.saturation-0.001)*(imagei!=conf.saturation)
		imagei_calib = (1.-test)*conf.saturation + test*imagei_calib
		imagei_calib = (imagei_calib>=0.)*imagei_calib						#this step is required for coalignment with star matching (for star detection) otherwise mean is too low due to values << 0		
		print 'dark and flat calibration ... done'

		#calculate shifts
		starsi = astro.find_stars_accurate(imagei_calib, nb_stars, detection_area, excluded=excl, included=incl)
		dx,dy = astro.find_shift_stars_matching(stars0, starsi, text_display=text)
		print 'shifts of image', str(i+1),' wrt image 1 (vert, hor):', dx, dy

		#shift image
		imagei_aligned = astro.translate_Bspline(imagei_calib, (-dx,-dy), 3)

		#save image
		headerdict = astro.create_headers(Config, [Imagelist[i]])[0]
		headerdict.update(Headers)
		astro.save_imagelist(imagei_aligned, [headerdict], [Imagelist2[i]])
		print Imagelist2[i] + ' saved'
	#-------------------------------------

def calibrate_coalign2_and_save(Imagelist, Darklist, Flat, Imagelist2, Config, Headers={}):

	#------ creating dark frame ------
	darks = astro.get_imagedata(Darklist)
	dark_final = astro.add_median(darks)
	hotpixels = calib7.find_hotpixels(dark_final, 500)
	
	#------ creating flat frame ------
	flat = astro.get_imagedata([Flat])
	flat = flat / astro.get_median(flat)
	flat = flat + 1e-10

	#------ calibration and co-registration of 1st image ------
	image0 = astro.get_imagedata([Imagelist[0]])
	image0_calib = (image0 - dark_final) / flat
	image0_avg = (np.roll(image0_calib,1,axis=0) + np.roll(image0_calib,-1,axis=0) + np.roll(image0_calib,1,axis=1) + np.roll(image0_calib,-1,axis=1)) / 4.
	image0_calib = image0_avg*hotpixels + image0_calib*(1.-hotpixels)

	#------ checking for saturation ------
	test = (image0_calib<conf.saturation-0.001)*(image0!=conf.saturation)
	image0_calib = (1.-test)*conf.saturation + test*image0_calib
	print 'dark and flat calibration ... done'
	print 'shifts (vert, hor) wrt reference image:', 0., 0.
	
	#save image
	headerdict = astro.create_headers(Config, [Imagelist[0]])[0]
	headerdict.update(Headers)
	astro.save_imagelist(image0_calib, [headerdict], [Imagelist2[0]])
	print Imagelist2[0] + ' saved'

	#------ same for all other images ------
	for i in range(1,len(Imagelist)):

		#loading and calibration
		imagei = astro.get_imagedata([Imagelist[i]])
		imagei_calib = (imagei - dark_final) / flat
		imagei_avg = (np.roll(imagei_calib,1,axis=0) + np.roll(imagei_calib,-1,axis=0) + np.roll(imagei_calib,1,axis=1) + np.roll(imagei_calib,-1,axis=1)) / 4.
		imagei_calib = imagei_avg*hotpixels + imagei_calib*(1.-hotpixels)

		#checking for saturation
		test = (imagei_calib<conf.saturation-0.001)*(imagei!=conf.saturation)
		imagei_calib = (1.-test)*conf.saturation + test*imagei_calib
		print 'dark and flat calibration ... done'

		#calculate shifts
		dx, dy = astro.find_shift_crosscorrelation(image0_calib, imagei_calib)
		print 'shifts of image', str(i+1),' wrt image 1 (vert, hor):', dx, dy

		#shift image
		imagei_aligned = astro.translate_Bspline(imagei_calib, (-dx,-dy), 3)

		#save image
		headerdict = astro.create_headers(Config, [Imagelist[i]])[0]
		headerdict.update(Headers)
		astro.save_imagelist(imagei_aligned, [headerdict], [Imagelist2[i]])
		print Imagelist2[i] + ' saved'
	#-------------------------------------
