# -*- coding: iso_8859_1 -*-

import os, datetime
import numpy as np

def write_auto_magnitudes(Imagelist, dateobs, xy, sizes, mag, stats, quality_flag, ref_stars, outputname):
	
	if os.path.exists(outputname):
		os.remove(outputname)
	f = open(outputname, 'w')

	#--- nb of stars to be written to file ---
	ind = list(np.where(np.array(quality_flag)>=0)[0])
	
	#--- file header ---
	f.write('#--- MAIN INFORMATION ------------------\n')
	f.write('date = '+str((datetime.datetime.now()).strftime("%Y-%m-%d %H:%M:%S"))+'\n')
	f.write(str(len(Imagelist))+' images processed\n')
	f.write(str(len(ind))+' objects successfully measured\n')
	f.write('first image = '+(Imagelist[0]).encode('iso_8859_1')+'\n')
	f.write('photometry with apertures:\t'+str(sizes[0])+' - '+str(sizes[1])+' - '+str(sizes[2])+'\n\n')

	#--- reference stars ---
	print ref_stars
	f.write('#--- REFERENCE STARS --\n')
	f.write('nb of ref stars:' + str(len(ref_stars)) + '\n')
	f.write('ref stars:')
	for ref in np.sort(ref_stars):
		f.write('\t' + str(ref))
	f.write('\n\n')

	#--- stars X,Y coordinates ---
	f.write('#--- OBJECTS COORDINATES ON FIRST FRAME --\n')
	f.write('Object nb')
	for j in range(len(ind)):
		f.write('\t'+str(j+1))
	f.write('\n')
	f.write('X position')
	for j in ind:
		f.write('\t'+str(xy[0][j][0]))
	f.write('\n')
	f.write('Y position')
	for j in ind:
		f.write('\t'+str(xy[0][j][1]))
	f.write('\n\n')

	#--- statistics about photometry ---
	signal, noise_global, noise_local = stats

	f.write('#--- STATISTICS ------------------------\n')
	f.write('mag signal')
	for j in ind:
		f.write('\t'+str(signal[j]))
	f.write('\n')
	f.write('global noise')
	for j in ind:
		f.write('\t'+str(noise_global[j]))
	f.write('\n')
	f.write('local noise')
	for j in ind:
		f.write('\t'+str(noise_local[j]))
	f.write('\n')
	f.write('quality flag')
	for j in ind:
		f.write('\t'+str(quality_flag[j]))
	f.write('\n\n')
	
	#--- measured magnitudes ---
	f.write('#--- JULIAN DAY + MEASURED MAGNITUDES ---------------\n')
	data = np.ndarray((len(dateobs),len(ind)+1))
	data[:,0] = np.array(dateobs)
	data[:,1:] = np.ma.filled(mag[:,ind],fill_value=np.nan)
	np.savetxt(f,data,delimiter='\t',fmt='%.6f')

	f.close()
	#-------------------------------------