# -*- coding: iso_8859_1 -*-

from aspylib import astro
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.cm as cm
import matplotlib.font_manager as fm

def check_refstars(names, mag):

	nb_obj = len([i for i in names if (i[0]=='m')or(i[0]=='v')])
	nb_ref = len([i for i in names if (i[0]=='r')])

	rejstar = []
	rejdata = []

	mask = np.ma.make_mask_none(np.shape(mag))
	mag_out = np.ma.masked_array(mag,mask=mask)

	#------ first check to reject wrong measurements --------------

	#removing reference star that are poorly measured
	for i in range(nb_obj,len(names)):		#loop on all reference stars
		test_refstar = np.sum(np.isnan(mag_out[:,i,:]).flatten())
		total = np.shape(mag_out)[0] * np.shape(mag_out)[2]
		if test_refstar >= 0.05*total:
			print "Object " + names[i] + " has more than 5% of wrong measurements, it is removed"
			rejstar.append(i)
			mag_out.mask[:,i,:]=True		

	#removing image for which remaining reference stars are not correctly measured
	for i in range(len(mag_out)):		#loop on all images
		test_images = np.sum(np.isnan(mag_out[i,nb_obj:,:]).flatten())
		if test_images > 0:
			print "Image " + str(i+1) + " has wrong measurements, it is removed"
			rejdata.append(i)
			mag_out.mask[i,:,:] = True

	#--> at this moment we have a set of images and reference stars free of any wrong measurement

	#------ second check based on user visual verification --------------
	if (nb_ref-len(rejstar)>1):	#the ref star check can be continued		
		plt.ion()
		while True:
			#------ plot instrument differential magnitudes for reference stars ------
			fig = plt.figure()
			fig.canvas.set_window_title("Reference stars")
			plt.axes([0.1,0.1,0.7,0.8])

			indices = ~(mag_out.mask[:,0,0])	#indexing unmasked images
			for j in range(nb_obj,len(names)):
				if j not in rejstar:
					plt.plot(mag_out[indices,j,0] - np.mean(mag_out[:,nb_obj:,0], axis=1)[indices], c=cm.Set1(j*256/(len(names)-nb_obj)), label=(names[j]))

			plt.legend(prop=fm.FontProperties(size=11), bbox_to_anchor=(1.02, 1.02), loc=2)
			plt.title("Differential magnitudes for REF stars")
			plt.grid()
			plt.draw()
			sx, sy = astro.get_window_size(u"C:\\WINDOWS\\system32\\cmd.exe")
			fx, fy = astro.get_window_position("Reference stars")
			astro.move_window("Reference stars", [sx, fy])
			astro.set_focus(u"C:\\WINDOWS\\System32\\cmd.exe")
			
			#------ question ---------------------------------------------------------
			print
			ans = raw_input("Are reference stars OK (answer must be 'yes' or 'no')?\n")
			plt.close()
				
			if (ans=='yes'):
				break
			else:
				#------ then plot instrumental magnitudes ----------------------------
				fig = plt.figure()
				fig.canvas.set_window_title("Reference stars")
				plt.axes([0.1,0.1,0.7,0.8])
			
				indices = ~(mag_out.mask[:,0,0])	#indexing unmasked images
				for j in range(nb_obj,len(names)):
					if j not in rejstar:
						plt.plot(mag_out[indices,j,0], label=(names[j]))

				plt.legend(prop=fm.FontProperties(size=11), bbox_to_anchor=(1.02, 1.02), loc=2)
				plt.grid()
				plt.title("Instrumental magnitudes for REF stars")
				plt.draw()
				sx, sy = astro.get_window_size(u"C:\\WINDOWS\\system32\\cmd.exe")
				fx, fy = astro.get_window_position("Reference stars")
				astro.move_window("Reference stars", [sx, fy])
				astro.set_focus(u"C:\\WINDOWS\\System32\\cmd.exe")

				while True:
					try:
						wrong = np.int(raw_input("which reference star is wrong ?\n"))
						break
					except:
						print "please enter an integer !"
				if (wrong>=1)and(wrong<=nb_ref):
					if (nb_obj+wrong-1) not in rejstar:
						rejstar.append(nb_obj+wrong-1)
						mag_out.mask[:,nb_obj+wrong-1,:] = True

			plt.close()

	return mag_out, rejstar, rejdata