# -*- coding: iso_8859_1 -*-

from aspylib import astro
import numpy as np

def make_photometry(Imagelist, sizes, coord, method='centroid', error_messages = 1):

	#loop on images
	print
	mag_total = []
	
	for i in range(len(Imagelist)):
		#loads relevant image
		print 'Photometry on :',Imagelist[i].split('\\')[-1]
		image = astro.get_imagedata([Imagelist[i]], text_display=False)

		mag_total.append(aperture_photometry(image, sizes, coord[i], method=method, error_messages = error_messages))
		
	return np.array(mag_total)

def aperture_photometry(image, sizes, coord, method='centroid', error_messages = 1):
	
	#-------------------------
	if method == 'centroid':
		maxsize = np.ceil(max(sizes)[0])
		image2 = np.array(image)
	#-------------------------
	
	error_image_border = []
	error_negative_intensity = []
		
	#loop on stars / asteroids
	mag_stars = []

	for j, xy in enumerate(coord):

		mag_methods = []

		if np.isnan(xy[0]):
			for k in range(len(sizes)):		
				mag_methods.append(np.nan)
		else:
			#coordinates for object center
			x = np.round(xy[0])
			y = np.round(xy[1])
			dx = xy[0] - x
			dy = xy[1] - y
		
			#-------------------------
			if method == 'centroid':
				xnotmax = max(0,x-maxsize-3)
				xrangemax = min(image.shape[0],x+maxsize+4)
				ynotmax = max(0,y-maxsize-3)
				yrangemax = min(image.shape[1],y+maxsize+4)
				image2[xnotmax:xrangemax,ynotmax:yrangemax] = astro.translate_Bspline(image2[xnotmax:xrangemax,ynotmax:yrangemax],(-dx,-dy),3)
			#-------------------------
		
			#loop on apertures for photometry
			for k in range(len(sizes)):		

				#-------------------------
				if method == 'centroid':
					test = (x-sizes[k][0]-3<0)or(x+sizes[k][0]+4>image.shape[0])or(y-sizes[k][0]-3<0)or(y+sizes[k][0]+4>image.shape[1])
				else:
					test = (x-sizes[k][0]<0)or(x+sizes[k][0]+1>image.shape[0])or(y-sizes[k][0]<0)or(y+sizes[k][0]+1>image.shape[1])
				#-------------------------

				if 	test:
					error_image_border.append([j+1,k+1])
					mag_methods.append(np.nan)
				else:
					#selecting subimage
					xnot = max(0,x-sizes[k][2])
					xrange = min(image.shape[0],x+sizes[k][2]+1)
					ynot = max(0,y-sizes[k][2])
					yrange = min(image.shape[1],y+sizes[k][2]+1)

					subimage = image[xnot:xrange,ynot:yrange]

					#aperture photometry
					grid = np.indices(subimage.shape)
					radius = (grid[0]-x+xnot)**2+(grid[1]-y+ynot)**2

					cond1 = radius<=sizes[k][0]**2
					cond2 = (radius>=sizes[k][1]**2)*(radius<=sizes[k][2]**2)

					#-------------------------
					if method == 'centroid':
						subimage2 = image2[xnot:xrange,ynot:yrange]
						center = np.extract(cond1,subimage2)
					else:
						center = np.extract(cond1,subimage)
					#-------------------------
					crown = np.extract(cond2,subimage)

					#print np.where(radius==np.min(np.min(radius)))
					#print np.where(subimage==np.max(np.max(subimage)))
					#raw_input("ok?")

					#------ magnitude calculation ------
					floor = astro.get_median(np.tile(np.array(crown,dtype=np.float),(1,1,1)))[0]
					intensity = np.sum(np.array(center,dtype=np.float)-floor)
					if intensity <= 0.:
						error_negative_intensity.append([j+1,k+1])
						mag_methods.append(np.nan)
					else:
						mag_methods.append(-2.5*np.log10(intensity))

			#-------------------------
			if method == 'centroid':
				image2[xnotmax:xrangemax,ynotmax:yrangemax] = image[xnotmax:xrangemax,ynotmax:yrangemax]
			#-------------------------
		mag_stars.append(mag_methods)
	
	if error_messages==1:
		if len(error_image_border)>0:
			print 'photometry.aperture_photometry():',len(error_image_border),'/',len(coord)*len(sizes),'measurement(s) on image border'
		if len(error_negative_intensity)>0:
			print 'photometry.aperture_photometry():',len(error_negative_intensity),'/',len(coord)*len(sizes),'measurement(s) with <0 intensity'

	elif error_messages==2:
		if len(error_image_border)>0:
			print 'photometry.aperture_photometry():',len(error_image_border),'/',len(coord)*len(sizes),'measurement(s) on image border:'
			print str(error_image_border)[1:-1]
		if len(error_negative_intensity)>0:
			print 'photometry.aperture_photometry():',len(error_negative_intensity),'/',len(coord)*len(sizes),'measurement(s) with <0 intensity:'
			print str(error_negative_intensity)[1:-1]

	return np.array(mag_stars)
